local LMP = LibStub("LibMediaProvider-1.0")

local MOUSEMENU_HEADER_TEXT_COLOR = "#ecdc6e"
local MOUSEMENU_BUTTON_TEXT_COLOR = "#ffffff"
local MOUSEMENU_BUTTON_TEXT_HIGHLIGHT_COLOR = "#f3aa6f"
local MOUSEMENU_BUTTON_HIGHLIGHT_COLOR = "#1d5783"

local MOUSEMENU_HEADER_FONT_SIZE = 18

local MOUSEMENU_MIN_WIDTH = 200
local MOUSEMENU_MIN_HEIGHT = 100

local mouseButtons = {}
local mouseLines = {}
local nextButtonTop = nil

local function SetMouseMenuDimensions(_width, height)
	AUI_MouseMenu:SetDimensions(_width, height)	
	AUI_MouseMenu_BG:SetDimensions(_width, height) 
	AUI_MouseMenu_Header:SetDimensions(_width, MOUSEMENU_HEADER_FONT_SIZE)
	AUI_MouseMenu_Header:SetDimensions(_width, MOUSEMENU_HEADER_FONT_SIZE)
	AUI_MouseMenu_HeaderLine:SetDimensions(_width, 2)
end

function AUI.AddMouseMenuButton(_name, _text, _func)
	local buttonHeight = 32
		
	local button =  mouseButtons[_name]
		
	if not button then	
		button = WINDOW_MANAGER:CreateControl(_name, AUI_MouseMenu, CT_CONTROL)			
		button:SetMouseEnabled(true)	
		
		button:SetHandler("OnMouseEnter", function(self)
			self.ButtonHighlight:SetHidden(false)
			self.Label:SetColor(AUI.Color.ConvertHexToRGBA(MOUSEMENU_BUTTON_TEXT_HIGHLIGHT_COLOR, 1.0):UnpackRGBA())
		end)	
		
		button:SetHandler("OnMouseExit", function(self)
			self.ButtonHighlight:SetHidden(true)
			self.Label:SetColor(AUI.Color.ConvertHexToRGBA(MOUSEMENU_BUTTON_TEXT_COLOR, 1.0):UnpackRGBA())
		end)			
		
		button.ButtonHighlight = WINDOW_MANAGER:CreateControl(nil, button, CT_TEXTURE)
		button.ButtonHighlight:SetAnchor(TOPLEFT, button, TOPLEFT, 0, 0)
		button.ButtonHighlight:SetColor(AUI.Color.ConvertHexToRGBA(MOUSEMENU_BUTTON_HIGHLIGHT_COLOR, 0.8):UnpackRGBA())
		button.ButtonHighlight:SetHidden(true)
		button.ButtonHighlight:SetMouseEnabled(false)	
		
		button.Label = WINDOW_MANAGER:CreateControl(nil, button, CT_LABEL)
		button.Label:SetColor(AUI.Color.ConvertHexToRGBA(MOUSEMENU_BUTTON_TEXT_COLOR, 1.0):UnpackRGBA())	

		button.Label:SetFont(LMP:Fetch('font', "Univers 67") .. "|" .. 16 .. "|" .. "outline")	
		button.Label:SetMouseEnabled(false)
		button.Label:SetHorizontalAlignment(TEXT_ALIGN_CENTER ) 	
		button.Label:SetVerticalAlignment(TEXT_ALIGN_CENTER ) 
		button.Label:SetWrapMode(TEXT_WRAP_MODE_ELLIPSIS ) 	
	end
	
	button:SetDimensions(MOUSEMENU_MIN_WIDTH, buttonHeight)
	button:SetAnchor(TOPLEFT, AUI_MouseMenu, TOPLEFT, 0, nextButtonTop)	
	button:SetHidden(false)
	button:SetHandler("OnMouseUp", function() 
												AUI.HideMouseMenu()
												_func(button) 
											 end
							    )
									
	nextButtonTop = (nextButtonTop + buttonHeight) + 4
	
	button.ButtonHighlight:SetDimensions(MOUSEMENU_MIN_WIDTH, buttonHeight)	
	
	button.Label:SetDimensions(MOUSEMENU_MIN_WIDTH, 30)
	button.Label:SetAnchor(CENTER, button, CENTER, 0, 0)	
	button.Label:SetText(_text)
	
	AUI_MouseMenu:SetDimensions(MOUSEMENU_MIN_WIDTH, nextButtonTop + 1)	
	AUI_MouseMenu_BG:SetDimensions(MOUSEMENU_MIN_WIDTH, nextButtonTop + 1)
	
	mouseButtons[_name] = button
end

function AUI.AddMouseMenuLine(_name)
	local line =  mouseLines[_name]
	
	if not line then
		line = WINDOW_MANAGER:CreateControl(_name, AUI_MouseMenu, CT_TEXTURE)
		line:SetTexture("esoui/art/tradinghouse/tradinghouse_divider_short.dds")			
	end
	
	line:SetAnchor(TOPCENTER, AUI_MouseMenu, TOPCENTER, 0, nextButtonTop)
	line:SetDimensions(MOUSEMENU_MIN_WIDTH, 2)	
	line:SetHidden(false)
	
	nextButtonTop = (nextButtonTop + 2) + 4
	mouseLines[_name] = line
end

function AUI.ShowMouseMenu(_displayName, _closeFunc)
	nextButtonTop = 36

	for _, _button in pairs(mouseButtons) do
		_button:SetHidden(true)
	end
	
	for _, _line in pairs(mouseLines) do
		_line:SetHidden(true)
	end	
	
	AUI_MouseMenu:SetHandler("OnMouseEnter", function(_eventCode, _button, _ctrl, _alt, shift) AUI_MouseMenu_Container:SetMouseEnabled(false) end)
	AUI_MouseMenu:SetHandler("OnMouseExit", function(_eventCode, _button, _ctrl, _alt, shift) AUI_MouseMenu_Container:SetMouseEnabled(true) end)
		
	AUI_MouseMenu_Header:SetColor(AUI.Color.ConvertHexToRGBA(MOUSEMENU_HEADER_TEXT_COLOR, 1.0):UnpackRGBA())	
	AUI_MouseMenu_Header:SetFont(LMP:Fetch('font', "Univers 67") .. "|" .. MOUSEMENU_HEADER_FONT_SIZE .. "|" .. "outline")	
	AUI_MouseMenu_Header:SetAnchor(TOPCENTER, AUI_MouseMenu, TOPCENTER, 0, 4)		
	AUI_MouseMenu_Header:SetHorizontalAlignment(TEXT_ALIGN_CENTER ) 	
	AUI_MouseMenu_Header:SetVerticalAlignment(TEXT_ALIGN_CENTER ) 		
		
	AUI_MouseMenu_HeaderLine:SetTexture("esoui/art/tradinghouse/tradinghouse_divider_short.dds")		
	AUI_MouseMenu_HeaderLine:SetAnchor(TOPCENTER, AUI_MouseMenu, TOPCENTER, 0, nextButtonTop)	
	AUI_MouseMenu_HeaderLine:SetHidden(false)
		
	AUI_MouseMenu_Container:ClearAnchors()	
	AUI_MouseMenu_Container:SetAnchorFill(GuiRoot)  
	AUI_MouseMenu_Container:SetHidden(false)	
	AUI_MouseMenu_Container:SetHandler("OnMouseDown", function(_eventCode, _button, _ctrl, _alt, shift) AUI.HideMouseMenu() end)
			
	SetMouseMenuDimensions(MOUSEMENU_MIN_WIDTH, MOUSEMENU_MIN_HEIGHT)
	
	nextButtonTop = (nextButtonTop + 2) + 2
	
	local x, y = GetUIMousePosition()
	AUI_MouseMenu:SetAnchor(TOPLEFT, GUIROOT, TOPLEFT, x, y)
	
	AUI_MouseMenu:SetHidden(false)
	
	AUI_MouseMenu_Header:SetText(_displayName)
end

function AUI.HideMouseMenu()
	AUI_MouseMenu:SetHidden(true)
	AUI_MouseMenu_Container:SetHidden(true)
end

EVENT_MANAGER:RegisterForEvent("ADVANCED_UI_EVENT_ACTION_LAYER_POPPED", EVENT_ACTION_LAYER_POPPED, AUI.HideMouseMenu)
EVENT_MANAGER:RegisterForEvent("ADVANCED_UI_EVENT_ACTION_LAYER_PUSHED", EVENT_ACTION_LAYER_PUSHED, AUI.HideMouseMenu)